@@ -147,8 +147,8 @@ public:
     void remove(int i, int n);
     inline void removeFirst() { Q_ASSERT(!isEmpty()); erase(d->begin()); }
     inline void removeLast();
-    inline T takeFirst() { Q_ASSERT(!isEmpty()); T r = first(); removeFirst(); return r; }
-    inline T takeLast()  { Q_ASSERT(!isEmpty()); T r = last(); removeLast(); return r; }
+    T takeFirst() { Q_ASSERT(!isEmpty()); T r = std::move(first()); removeFirst(); return r; }
+    T takeLast()  { Q_ASSERT(!isEmpty()); T r = std::move(last()); removeLast(); return r; }
 
     QVector<T> &fill(const T &t, int size = -1);
 
@@ -164,9 +164,10 @@ public:
         const const_iterator ce = this->cend(), cit = std::find(this->cbegin(), ce, t);
         if (cit == ce)
             return 0;
-        // next operation detaches, so ce, cit may become invalidated:
+        // next operation detaches, so ce, cit, t may become invalidated:
+        const T tCopy = t;
         const int firstFoundIdx = std::distance(this->cbegin(), cit);
-        const iterator e = end(), it = std::remove(begin() + firstFoundIdx, e, t);
+        const iterator e = end(), it = std::remove(begin() + firstFoundIdx, e, tCopy);
         const int result = std::distance(it, e);
         erase(it, e);
         return result;
@@ -180,7 +181,7 @@ public:
         return true;
     }
     int length() const { return size(); }
-    T takeAt(int i) { T t = at(i); remove(i); return t; }
+    T takeAt(int i) { T t = std::move((*this)[i]); remove(i); return t; }
     void move(int from, int to)
     {
         Q_ASSERT_X(from >= 0 && from < size(), "QVector::move(int,int)", "'from' is out-of-range");
@@ -268,6 +269,7 @@ public:
     inline const_reference front() const { return first(); }
     inline reference back() { return last(); }
     inline const_reference back() const { return last(); }
+    void shrink_to_fit() { squeeze(); }
 
     // comfort
     QVector<T> &operator+=(const QVector<T> &l);
@@ -291,6 +293,7 @@ public:
 private:
     friend class QRegion; // Optimization for QRegion::rects()
 
+    // ### Qt6: remove const from int parameters
     void reallocData(const int size, const int alloc, QArrayData::AllocationOptions options = QArrayData::Default);
     void reallocData(const int sz) { reallocData(sz, d->alloc); }
     void freeData(Data *d);
@@ -309,6 +312,7 @@ private:
 // will be default-initialized
 #   pragma warning ( push )
 #   pragma warning ( disable : 4345 )
+#   pragma warning(disable : 4127) // conditional expression is constant
 #endif
 
 template <typename T>
@@ -323,10 +327,6 @@ void QVector<T>::defaultConstruct(T *from, T *to)
     }
 }
 
-#ifdef Q_CC_MSVC
-#   pragma warning ( pop )
-#endif
-
 template <typename T>
 void QVector<T>::copyConstruct(const T *srcFrom, const T *srcTo, T *dstFrom)
 {
@@ -338,11 +338,6 @@ void QVector<T>::copyConstruct(const T *srcFrom, const T *srcTo, T *dstFrom)
     }
 }
 
-#if defined(Q_CC_MSVC)
-#pragma warning( push )
-#pragma warning( disable : 4127 ) // conditional expression is constant
-#endif
-
 template <typename T>
 void QVector<T>::destruct(T *from, T *to)
 {
@@ -353,10 +348,6 @@ void QVector<T>::destruct(T *from, T *to)
     }
 }
 
-#if defined(Q_CC_MSVC)
-#pragma warning( pop )
-#endif
-
 template <typename T>
 inline QVector<T>::QVector(const QVector<T> &v)
 {
@@ -378,6 +369,10 @@ inline QVector<T>::QVector(const QVector<T> &v)
     }
 }
 
+#if defined(Q_CC_MSVC)
+#pragma warning( pop )
+#endif
+
 template <typename T>
 void QVector<T>::detach()
 {
@@ -505,6 +500,11 @@ QVector<T>::QVector(int asize, const T &t)
 }
 
 #ifdef Q_COMPILER_INITIALIZER_LISTS
+# if defined(Q_CC_MSVC)
+QT_WARNING_PUSH
+QT_WARNING_DISABLE_MSVC(4127) // conditional expression is constant
+# endif // Q_CC_MSVC
+
 template <typename T>
 QVector<T>::QVector(std::initializer_list<T> args)
 {
@@ -519,7 +519,10 @@ QVector<T>::QVector(std::initializer_list<T> args)
         d = Data::sharedNull();
     }
 }
-#endif
+# if defined(Q_CC_MSVC)
+QT_WARNING_POP
+# endif // Q_CC_MSVC
+#endif // Q_COMPILER_INITALIZER_LISTS
 
 template <typename T>
 void QVector<T>::freeData(Data *x)
@@ -528,6 +531,11 @@ void QVector<T>::freeData(Data *x)
     Data::deallocate(x);
 }
 
+#if defined(Q_CC_MSVC)
+QT_WARNING_PUSH
+QT_WARNING_DISABLE_MSVC(4127) // conditional expression is constant
+#endif
+
 template <typename T>
 void QVector<T>::reallocData(const int asize, const int aalloc, QArrayData::AllocationOptions options)
 {
@@ -553,7 +561,7 @@ void QVector<T>::reallocData(const int asize, const int aalloc, QArrayData::Allo
                 T *srcEnd = asize > d->size ? d->end() : d->begin() + asize;
                 T *dst = x->begin();
 
-                if (QTypeInfo<T>::isStatic || (isShared && QTypeInfo<T>::isComplex)) {
+                if (!QTypeInfoQuery<T>::isRelocatable || (isShared && QTypeInfo<T>::isComplex)) {
                     // we can not move the data, we need to copy construct it
                     while (srcBegin != srcEnd) {
                         new (dst++) T(*srcBegin++);
@@ -598,7 +606,7 @@ void QVector<T>::reallocData(const int asize, const int aalloc, QArrayData::Allo
     }
     if (d != x) {
         if (!d->ref.deref()) {
-            if (QTypeInfo<T>::isStatic || !aalloc || (isShared && QTypeInfo<T>::isComplex)) {
+            if (!QTypeInfoQuery<T>::isRelocatable || !aalloc || (isShared && QTypeInfo<T>::isComplex)) {
                 // data was copy constructed, we need to call destructors
                 // or if !alloc we did nothing to the old 'd'.
                 freeData(d);
@@ -619,6 +627,10 @@ void QVector<T>::reallocData(const int asize, const int aalloc, QArrayData::Allo
     Q_ASSERT(d->size == asize);
 }
 
+#if defined(Q_CC_MSVC)
+QT_WARNING_POP
+#endif
+
 template<typename T>
 Q_OUTOFLINE_TEMPLATE T QVector<T>::value(int i) const
 {
@@ -692,12 +704,12 @@ typename QVector<T>::iterator QVector<T>::insert(iterator before, size_type n, c
 {
     Q_ASSERT_X(isValidIterator(before),  "QVector::insert", "The specified iterator argument 'before' is invalid");
 
-    int offset = std::distance(d->begin(), before);
+    const auto offset = std::distance(d->begin(), before);
     if (n != 0) {
         const T copy(t);
         if (!isDetached() || d->size + n > int(d->alloc))
             reallocData(d->size, d->size + n, QArrayData::Grow);
-        if (QTypeInfo<T>::isStatic) {
+        if (!QTypeInfoQuery<T>::isRelocatable) {
             T *b = d->end();
             T *i = d->end() + n;
             while (i != b)
@@ -728,7 +740,7 @@ typename QVector<T>::iterator QVector<T>::erase(iterator abegin, iterator aend)
     Q_ASSERT_X(isValidIterator(abegin), "QVector::erase", "The specified iterator argument 'abegin' is invalid");
     Q_ASSERT_X(isValidIterator(aend), "QVector::erase", "The specified iterator argument 'aend' is invalid");
 
-    const int itemsToErase = aend - abegin;
+    const auto itemsToErase = aend - abegin;
 
     if (!itemsToErase)
         return abegin;
@@ -737,7 +749,7 @@ typename QVector<T>::iterator QVector<T>::erase(iterator abegin, iterator aend)
     Q_ASSERT(aend <= d->end());
     Q_ASSERT(abegin <= aend);
 
-    const int itemsUntouched = abegin - d->begin();
+    const auto itemsUntouched = abegin - d->begin();
 
     // FIXME we could do a proper realloc, which copy constructs only needed data.
     // FIXME we are about to delete data - maybe it is good time to shrink?
@@ -746,7 +758,7 @@ typename QVector<T>::iterator QVector<T>::erase(iterator abegin, iterator aend)
         detach();
         abegin = d->begin() + itemsUntouched;
         aend = abegin + itemsToErase;
-        if (QTypeInfo<T>::isStatic) {
+        if (!QTypeInfoQuery<T>::isRelocatable) {
             iterator moveBegin = abegin + itemsToErase;
             iterator moveEnd = d->end();
             while (moveBegin != moveEnd) {
@@ -766,7 +778,7 @@ typename QVector<T>::iterator QVector<T>::erase(iterator abegin, iterator aend)
             memmove(static_cast<void *>(abegin), static_cast<void *>(aend),
                     (d->size - itemsToErase - itemsUntouched) * sizeof(T));
         }
-        d->size -= itemsToErase;
+        d->size -= int(itemsToErase);
     }
     return d->begin() + itemsUntouched;
 }
@@ -993,6 +1005,8 @@ Q_TEMPLATE_EXTERN template class Q_CORE_EXPORT QVector<QPointF>;
 Q_TEMPLATE_EXTERN template class Q_CORE_EXPORT QVector<QPoint>;
 #endif
 
+QVector<uint> QStringView::toUcs4() const { return QtPrivate::convertToUcs4(*this); }
+
 QT_END_NAMESPACE
 
 #endif // QVECTOR_H
